@@ -3,7 +3,6 @@ from time import sleep
 from sys import maxsize
 
 
-
 class RestreamedBytesIO(object):
     __slots__ = ["substream", "encoder", "encoderunit", "decoder", "decoderunit", "rbuffer", "wbuffer","sincereadwritten"]
 
@@ -21,7 +20,10 @@ class RestreamedBytesIO(object):
         if count < 0:
             raise ValueError("count cannot be negative")
         while len(self.rbuffer) < count:
-            self.rbuffer += self.decoder(self.substream.read(self.decoderunit))
+            data = self.substream.read(self.decoderunit)
+            if data is None or len(data) == 0:
+                raise IOError("Restreamed cannot satisfy read request of %d bytes" % (count,))
+            self.rbuffer += self.decoder(data)
         data, self.rbuffer = self.rbuffer[:count], self.rbuffer[count:]
         self.sincereadwritten += count
         return data
@@ -52,8 +54,6 @@ class RestreamedBytesIO(object):
         return True
 
 
-
-
 class RebufferedBytesIO(object):
     __slots__ = ["substream","offset","rwbuffer","moved","tailcutoff"]
 
@@ -130,46 +130,3 @@ class RebufferedBytesIO(object):
 
     def cachedto(self):
         return self.moved + len(self.rwbuffer)
-
-
-
-class BoundBytesIO(object):
-    __slots__ = ["substream","start","size"]
-    def __init__(self, substream, available):
-        self.substream = substream
-        self.start = self.tell()
-        self.size = available
-
-    @property
-    def available(self):
-        return self.size - (self.tell() - self.start)
-
-    @property
-    def offset(self):
-        return self.substream.tell()
-
-    def read(self, count=maxsize):
-        avail = self.available
-        if avail == 0:
-            return b""
-        count = min(count, avail)
-        data = self.substream.read(count)
-        if len(data) < count:
-            raise IOError("could only read %s bytes, requested %s" % (len(data),count))
-        return data
-
-    def seekable(self):
-        return True
-
-    def seek(self, offset):
-        if offset < self.start or offset > self.start + self.size:
-            raise IOError("trying to seek out of bounds [%d-%d)" % (self.start, self.start + self.size))
-        self.substream.seek(offset)
-
-    def tell(self):
-        return self.offset
-
-    def tellable(self):
-        return True
-
-
