@@ -1,13 +1,8 @@
-"""
-Various containers exposed to the user.
-"""
-
 from construct.lib.py3compat import *
 
 
 globalfullprinting = None
 
-
 def setglobalfullprinting(enabled):
     r"""
     Sets full printing for all Container instances. When enabled, Container str produces full content of bytes and strings, otherwise and by default, it produces truncated output.
@@ -17,7 +12,6 @@ def setglobalfullprinting(enabled):
     global globalfullprinting
     globalfullprinting = enabled
 
-
 def getglobalfullprinting():
     """Used internally."""
     return bool(globalfullprinting)
@@ -50,7 +44,7 @@ class Container(dict):
     Example::
 
         Container([ ("name","anonymous"), ("age",21) ])
-        
+
         Container(name="anonymous")(age=21)
 
         # Note that this syntax does NOT work before python 3.6 due to unordered keyword arguments:
@@ -58,7 +52,7 @@ class Container(dict):
 
         Container(container2)
     """
-    __slots__ = ["__keys_order__","__recursion_lock__"]
+    __slots__ = ["__keys_order__", "__recursion_lock__"]
 
     def __init__(self, *args, **kw):
         object.__setattr__(self, "__keys_order__", [])
@@ -77,27 +71,45 @@ class Container(dict):
             self[k] = v
 
     def __getstate__(self):
-        return list(self.items())
+        return self.__keys_order__
 
     def __setstate__(self, state):
-        object.__setattr__(self, "__keys_order__", [])
-        dict.clear(self)
-        self.update(state)
+        self.__keys_order__ = state
 
     def __getattr__(self, name):
         try:
-            return self[name]
+            if name in self.__slots__:
+                try:
+                    return object.__getattribute__(self, name)
+                except AttributeError as e:
+                    if name == "__keys_order__":
+                        object.__setattr__(self, "__keys_order__", [])
+                        return []
+                    else:
+                        raise e
+            else:
+                return self[name]
         except KeyError:
             raise AttributeError(name)
 
     def __setitem__(self, key, val):
-        if key not in self:
-            self.__keys_order__.append(key)
-        dict.__setitem__(self, key, val)
+        if key in self.__slots__:
+            object.__setattr__(self, key, val)
+        else:
+            if key not in self:
+                if not hasattr(self, "__keys_order__"):
+                    object.__setattr__(self, "__keys_order__", [key])
+                else:
+                    self.__keys_order__.append(key)
+            dict.__setitem__(self, key, val)
 
     def __delitem__(self, key):
-        dict.__delitem__(self, key)
-        self.__keys_order__.remove(key)
+        """Removes an item from the Container in linear time O(n)."""
+        if key in self.__slots__:
+            object.__delattr__(self, key)
+        else:
+            dict.__delitem__(self, key)
+            self.__keys_order__.remove(key)
 
     __delattr__ = __delitem__
     __setattr__ = __setitem__
@@ -153,6 +165,10 @@ class Container(dict):
 
     __iter__ = keys
 
+    def __dir__(self):
+        """For auto completion of attributes based on container values."""
+        return list(self.keys()) + list(self.__class__.__dict__) + dir(super(Container, self))
+
     def __eq__(self, other):
         if not isinstance(other, dict):
             return False
@@ -166,22 +182,22 @@ class Container(dict):
                 return False
         return True
 
-    def _search(self, name, search_all):
+    def _search(self, compiled_pattern, search_all):
         items = []
         for key in self.keys():
             try:
-                if key == name:
-                    if search_all:
-                        items.append(self[key])
-                    else:
-                        return self[key]
                 if type(self[key]) == Container or type(self[key]) == ListContainer:
-                    ret = self[key]._search(name, search_all)
+                    ret = self[key]._search(compiled_pattern, search_all)
                     if ret is not None:
                         if search_all:
                             items.extend(ret)
                         else:
                             return ret
+                elif compiled_pattern.match(key):
+                    if search_all:
+                        items.append(self[key])
+                    else:
+                        return self[key]
             except:
                 pass
         if search_all:
@@ -189,11 +205,15 @@ class Container(dict):
         else:
             return None
 
-    def search(self, name):
-        return self._search(name, False)
+    def search(self, pattern):
+        import re
+        compiled_pattern = re.compile(pattern)
+        return self._search(compiled_pattern, False)
 
-    def search_all(self, name):
-        return self._search(name, True)
+    def search_all(self, pattern):
+        import re
+        compiled_pattern = re.compile(pattern)
+        return self._search(compiled_pattern, True)
 
     @recursion_lock()
     def __repr__(self):
@@ -213,11 +233,11 @@ class Container(dict):
         for k,v in self.items():
             if not isinstance(k,str) or not k.startswith("_"):
                 text.extend([indentation, str(k), " = "])
-                if isinstance(v, stringtypes) and fullprinting:
-                    if len(v) <= printingcap:
-                        text.append("%s (total %d)" % (v[:printingcap], len(v)))
+                if isinstance(v, stringtypes):
+                    if len(v) <= printingcap or fullprinting:
+                        text.append("%s (total %d)" % (reprbytes(v), len(v)))
                     else:
-                        text.append("%s... (truncated, total %d)" % (v[:printingcap], len(v)))
+                        text.append("%s... (truncated, total %d)" % (reprbytes(v[:printingcap]), len(v)))
                 else:
                     text.append(indentation.join(str(v).split("\n")))
         return "".join(text)
@@ -253,11 +273,11 @@ class ListContainer(list):
             text.append(indentation.join(lines))
         return "".join(text)
 
-    def _search(self, name, search_all):
+    def _search(self, compiled_pattern, search_all):
         items = []
         for item in self:
             try:
-                ret = item._search(name, search_all)
+                ret = item._search(compiled_pattern, search_all)
             except:
                 continue
             if ret is not None:
@@ -270,11 +290,15 @@ class ListContainer(list):
         else:
             return None
 
-    def search(self, name):
-        return self._search(name, False)
+    def search(self, pattern):
+        import re
+        compiled_pattern = re.compile(pattern)
+        return self._search(compiled_pattern, False)
 
-    def search_all(self, name):
-        return self._search(name, True)
+    def search_all(self, pattern):
+        import re
+        compiled_pattern = re.compile(pattern)
+        return self._search(compiled_pattern, True)
 
 
 class LazyContainer(object):
@@ -374,7 +398,6 @@ class LazyRangeContainer(ListContainer):
 
     def __repr__(self):
         return "<%s: %d possible items, %d cached>" % (self.__class__.__name__, len(self), len(self.cached))
-        # return "<%s: %s>" % (self.__class__.__name__, ",".join(repr(e) for e in self))
 
 
 class LazySequenceContainer(LazyRangeContainer):
@@ -405,5 +428,3 @@ class LazySequenceContainer(LazyRangeContainer):
 
     def __len__(self):
         return self.count
-
-
